#!/usr/bin/python

import numpy
from scipy.optimize import curve_fit
import matplotlib.pyplot as plt
import pdb
import sys


############################################################################

def calc_alpha_error(alpha1, alpha2, crtt_fixed_lambda):
  """
  calc_alpha_error calculates the timing error between the PPS-ses of Master
  and Slave that will be the result of an "imperfect" alpha2 with respect to a
  "perfect" alpha1.
  The timing error scales linear with the round trip time (i.e. the length of
  your fiber)

  alpha1            -- <float> alpha to be considered the correct value
  alpha2            -- <float> alpha for which timing error needs to be calculated
  crtt_fixed_lambda -- <float> crtt value at the fixed wavelength.
                               note that this is the virtual crtt for which
                               master2slave and slave2master wavelengths would be
                               equal. It can be estimated by extrapolating the crtt
                               array.

  returns:
  t_err             -- <float> PPS time error

  """

  t_err = ((alpha1-alpha2)*crtt_fixed_lambda)/4

  return(t_err)

############################################################################

def calc_alpha_error_stddev(alpha1, alpha1_stddev, alpha2, alpha2_stddev, crtt_fixed_lambda, crtt_fixed_lambda_stddev):
  """
  calc_alpha_error calculates the timing error between the PPS-ses of Master
  and Slave that will be the result of an "imperfect" alpha2 with respect to a
  "perfect" alpha1.
  The timing error scales linear with the round trip time (i.e. the length of
  your fiber)

  alpha1            -- <float> alpha to be considered the correct value
  alpha1_stddev     -- <float> stddev for alpha1
  alpha2            -- <float> alpha for which timing error needs to be calculated
  alpha2_stddev     -- <float> stddev for alpha2
  crtt_fixed_lambda -- <float> crtt value at the fixed wavelenght.
                               note that this is the virtual crtt for which
                               master2slave and slave2master wavelengths would be
                               equal. It can be estimated by extrapolating the crtt
                               array.
  crtt_fixed_lambda_stddev  -- <float> stddev for crtt_fixed_lambda

  returns:
  t_err, t_err_stddev -- <float> PPS time error

  """

  # Calculate the (mean) value of t_err
  t_err = calc_alpha_error(alpha1, alpha2, crtt_fixed_lambda)

  # Calculate the stddev contribution to t_err of each of the parameters alpha1, alpha2 and crtt_fixed_lambda
  t_err_stddev_alpha1       = calc_alpha_error(alpha1 + alpha1_stddev, alpha2, crtt_fixed_lambda) - t_err
  t_err_stddev_alpha2       = calc_alpha_error(alpha1, alpha2 + alpha2_stddev, crtt_fixed_lambda) - t_err
  t_err_stddev_crtt_fixed_lambda       = calc_alpha_error(alpha1, alpha2, crtt_fixed_lambda + crtt_fixed_lambda_stddev) - t_err

  # Calculate the RMS stddev of alpha
  t_err_stddev = numpy.sqrt(t_err_stddev_alpha1**2 + t_err_stddev_alpha2 **2 + t_err_stddev_crtt_fixed_lambda **2)
 
  return(t_err, t_err_stddev)

############################################################################

############################################################################

if __name__ == "__main__":

  #############################################################################################
  ## Determine the calculated time error (time_err_calc_3wl) due to the difference of the
  ## "perfect" alpha (derived from the Sellmeier fit) and the imperfect alpha that was determined
  ## using the 3-Lambda probed CRTT measurements.
  #############################################################################################

  # data source: data_180914/averaged_err.out => crtt_(l1)
  alpha_sel = numpy.array([6.424e-06, 6.424e-06, 3.207e-06, 3.207e-06, 3.207e-06, -3.197e-06, -3.197e-06, -3.197e-06, -6.384e-06, -6.384e-06, 2.922e-05, 2.922e-05, 2.922e-05, -2.529e-05, -2.529e-05, -2.529e-05, -7.391e-05, -7.391e-05, -7.391e-05, -1.226e-04, -1.226e-04, -1.226e-04])
  alpha_sel_stddev = numpy.array([1.682e-08, 1.682e-08, 1.674e-08, 1.674e-08, 1.674e-08, 1.671e-08, 1.671e-08, 1.671e-08, 1.675e-08, 1.675e-08, 1.843e-08, 1.843e-08, 1.843e-08, 1.786e-08, 1.786e-08, 1.786e-08, 2.600e-08, 2.600e-08, 2.600e-08, 3.850e-08, 3.850e-08, 3.850e-08])
  # data source: data_180926/180926_15_47_01_3_lambda_insitu_alpha_scan_result.txt
  alpha_3wl_b = numpy.array([6.393e-06, 6.449e-06, 3.154e-06, 3.262e-06, 3.214e-06, -3.224e-06, -3.193e-06, -3.298e-06, -6.488e-06, -6.348e-06, 2.898e-05, 2.847e-05, 2.788e-05, -2.551e-05, -2.454e-05, -2.402e-05, -7.464e-05, -7.303e-05, -7.009e-05, -1.238e-04, -1.211e-04, -1.188e-04])
  alpha_3wl_b_stddev = numpy.array([1.186e-06, 1.970e-07, 2.856e-07, 2.100e-07, 1.185e-07, 1.181e-07, 1.576e-07, 3.871e-07, 2.387e-07, 6.540e-07, 1.858e-07, 1.025e-07, 6.481e-08, 1.591e-07, 1.510e-07, 8.868e-08, 3.042e-07, 4.437e-07, 5.724e-07, 2.455e-07, 3.260e-07, 7.897e-07])

  # data source: data_180914/averaged_err.out => crtt_(l1)
  crtt_fixed_lambda = numpy.array([491284316, 491284316, 491283526, 491283526, 491283526, 491281953, 491281953, 491281953, 491281170, 491281170, 491289916, 491289916, 491289916, 491276526, 491276526, 491276526, 491264582, 491264582, 491264582, 491252613, 491252613, 491252613])
  crtt_fixed_lambda_stddev = numpy.array([124, 124, 124, 124, 124, 125, 125, 125, 125, 125, 123, 123, 123, 126, 126, 126, 129, 129, 129, 133, 133, 133])

  # calculate time error of the 3-wavelength determined alpha while taking the Sellmeier determined alpha as a reference.
  #time_err_calc_3wl = calc_alpha_error(alpha_sel, alpha_3wl_b, crtt_fixed_lambda)
  time_err_calc_3wl, time_err_calc_3wl_stddev = calc_alpha_error_stddev(alpha_sel, alpha_sel_stddev, alpha_3wl_b, alpha_3wl_b_stddev, crtt_fixed_lambda, crtt_fixed_lambda_stddev)
  result_file = open(("time_error_result.txt"),"w")
  result_file.write("time_err_calc_3wl, time_err_calc_3wl_stddev\n")

  for i in range(len(time_err_calc_3wl)):
    result_file.write("{0:.1f}".format(time_err_calc_3wl[i]) + ", " + "{0:.1f}".format(time_err_calc_3wl_stddev[i]) + "\n")
  result_file.close()
  
  ##############################################################
  #### Small wavelength offset
  ##############################################################

  #############################################################################################
  ## This piece of code plots the calculated time error (time_err_calc_sel) due to the difference of
  ## the "perfect" alpha (derived from the Sellmeier fit) and the imperfect alpha derived from
  ## the 3-lambda calculations on the same data set.
  ## (time_err_a, time_err_a_stddev) are the actual measured time errors due to this imperfect
  ## alpha.
  ## (time_err_b, time_err_b_stddev) are the actual measured time errors due to an imperfect
  ## alpha that was determined using the 3-Lambda probed CRTT measurements.
  #############################################################################################

  ## Small Lambda selection
  l1_selection = [7, 7, 8, 8, 8, 9, 9, 9, 10, 10]
  l2_selection = [8, 9, 7, 9, 10, 7, 8, 10, 8, 9] # determines color
  ## Full C-band selection
  #l1_cband_selection = [0, 0, 0, 16, 16, 16, 32, 32, 32, 49, 49, 49]
  #l2_cband_selection = [16, 32, 49, 0, 32, 49, 0, 16, 49, 0, 16, 32] # determines color
  ## concatenate both
  #l1_selection = l1_small_selection + l1_cband_selection
  #l2_selection = l2_small_selection + l2_cband_selection
  
  length = 10
  x = numpy.array(range(1, length+1))
  # time_err_calc_sel (i.e the calculated time error)
  # data source: p:\App\ASTERICS_Cleopatra_WP5_1\WR_Calibration\wr-calibration\sw\insitu_alpha\data_180914\averaged_err.out
  time_err_calc_sel = numpy.array([-0.82, 0.82, -0.82, 0.41, 0.82, 0.82, 0.41, -0.82, 0.82, -0.82])

  # time_meas_sel, time_meas_sel_stddev (i.e. PPS diff results)
  # data source ./calc_diff.py data_180703_combined/00_000_180703_00_00_00_Combined_DeltaDelay_meas data_180629_combined/00_000_180627_00_00_00_Combined_DeltaDelay_meas -t 600 -o data_180914_pps
  # => data_180914_pps/190409_10_07_48_DeltaDelayDifference
  time_err_a = numpy.array([14.3, 2.1, 4.0, 8.0, 8.7, 9.3, 4.3, 13.3, 22.4, 10.6])
  time_err_a_stddev = numpy.array([57.2, 50.1, 59.8, 61.2, 61.4, 60.5, 59.0, 58.2, 59.7, 55.6])
  
  # time_meas_3wl, time_meas_3wl_stddev (i.e. PPS diff results)
  # data source ./calc_diff.py data_180926/181030_15_19_37_DeltaDelay_meas data_180926/181031_09_12_18_DeltaDelay_meas -t 400 -o data_180914_pps
  # => data_180914_pps/190409_11_25_10_DeltaDelayDifference
  time_err_b = numpy.array([146.9, 59.3, 75.4, 28.0, 73.7, 26.9, 42.1, 55.9, 37.6, 54.4])
  time_err_b_stddev = numpy.array([50.0, 47.5, 62.4, 60.0, 67.6, 65.4, 43.9, 29.9, 41.7, 56.5])

  selected_x_0 = []
  selected_time_err_a_0 = []
  selected_time_err_a_0_stddev = []
  selected_time_err_b_0 = []
  selected_time_err_b_0_stddev = []
  selected_x_7 = []
  selected_time_err_a_7 = []
  selected_time_err_a_7_stddev = []
  selected_time_err_b_7 = []
  selected_time_err_b_7_stddev = []
  selected_x_8 = []
  selected_time_err_a_8 = []
  selected_time_err_a_8_stddev = []
  selected_time_err_b_8 = []
  selected_time_err_b_8_stddev = []
  selected_x_9 = []
  selected_time_err_a_9 = []
  selected_time_err_a_9_stddev = []
  selected_time_err_b_9 = []
  selected_time_err_b_9_stddev = []
  selected_x_10 = []
  selected_time_err_a_10 = []
  selected_time_err_a_10_stddev = []
  selected_time_err_b_10 = []
  selected_time_err_b_10_stddev = []
  selected_x_16 = []
  selected_time_err_a_16 = []
  selected_time_err_a_16_stddev = []
  selected_time_err_b_16 = []
  selected_time_err_b_16_stddev = []
  selected_x_32 = []
  selected_time_err_a_32 = []
  selected_time_err_a_32_stddev = []
  selected_time_err_b_32 = []
  selected_time_err_b_32_stddev = []
  selected_x_49 = []
  selected_time_err_a_49 = []
  selected_time_err_a_49_stddev = []
  selected_time_err_b_49 = []
  selected_time_err_b_49_stddev = []
  
  for i in range(length):
    if l2_selection[i] == 0:
      selected_x_0.append(i + 1)
      selected_time_err_a_0.append(time_err_a[i])
      selected_time_err_a_0_stddev.append(time_err_a_stddev [i])
      selected_time_err_b_0.append(time_err_b[i])
      selected_time_err_b_0_stddev.append(time_err_b_stddev[i])
    elif l2_selection[i] == 7:
      #print("hit 7")
      selected_x_7.append(i + 1)
      selected_time_err_a_7.append(time_err_a[i])
      selected_time_err_a_7_stddev.append(time_err_a_stddev [i])
      selected_time_err_b_7.append(time_err_b[i])
      selected_time_err_b_7_stddev.append(time_err_b_stddev[i])
    elif l2_selection[i] == 8:
      #print("hit 8")
      selected_x_8.append(i + 1)
      selected_time_err_a_8.append(time_err_a[i])
      selected_time_err_a_8_stddev.append(time_err_a_stddev [i])
      selected_time_err_b_8.append(time_err_b[i])
      selected_time_err_b_8_stddev.append(time_err_b_stddev[i])
    elif l2_selection[i] == 9:
      #print("hit 9")
      selected_x_9.append(i + 1)
      selected_time_err_a_9.append(time_err_a[i])
      selected_time_err_a_9_stddev.append(time_err_a_stddev [i])
      selected_time_err_b_9.append(time_err_b[i])
      selected_time_err_b_9_stddev.append(time_err_b_stddev[i])
    elif l2_selection[i] == 10:
      #print("hit 10")
      selected_x_10.append(i + 1)
      selected_time_err_a_10.append(time_err_a[i])
      selected_time_err_a_10_stddev.append(time_err_a_stddev [i])
      selected_time_err_b_10.append(time_err_b[i])
      selected_time_err_b_10_stddev.append(time_err_b_stddev[i])
    elif l2_selection[i] == 16:
      selected_x_16.append(i + 1)
      selected_time_err_a_16.append(time_err_a[i])
      selected_time_err_a_16_stddev.append(time_err_a_stddev [i])
      selected_time_err_b_16.append(time_err_b[i])
      selected_time_err_b_16_stddev.append(time_err_b_stddev[i])
    elif l2_selection[i] == 32:
      selected_x_32.append(i + 1)
      selected_time_err_a_32.append(time_err_a[i])
      selected_time_err_a_32_stddev.append(time_err_a_stddev [i])
      selected_time_err_b_32.append(time_err_b[i])
      selected_time_err_b_32_stddev.append(time_err_b_stddev[i])
    elif l2_selection[i] == 49:
      selected_x_49.append(i + 1)
      selected_time_err_a_49.append(time_err_a[i])
      selected_time_err_a_49_stddev.append(time_err_a_stddev [i])
      selected_time_err_b_49.append(time_err_b[i])
      selected_time_err_b_49_stddev.append(time_err_b_stddev[i])
 
  lns = []
  lns_a = []
  lns_b = []
  fig = plt.figure("Time_error")
  ax = fig.add_subplot(111)
  ax.set_title("Small wavelength offset calculated and measured time errors")
  ax.set_ylabel("Time error [ps]")
  ax.set_xlabel("measurement number")

  #lns_a.append(ax.errorbar(selected_x_0,  selected_time_err_a_0,  yerr = selected_time_err_a_0_stddev,  fmt='^', capsize=6, color='C1',    label=r'$t_{err\_m}$ A, $\lambda_{2}$ =1568.77 [nm]'))
  #lns_a.append(ax.errorbar(selected_x_16, selected_time_err_a_16, yerr = selected_time_err_a_16_stddev, fmt='^', capsize=6, color='C2',   label=r'$t_{err\_m}$ A, $\lambda_{2}$ =1554.94 [nm]'))
  #lns_a.append(ax.errorbar(selected_x_32, selected_time_err_a_32, yerr = selected_time_err_a_32_stddev, fmt='^', capsize=6, color='C3',   label=r'$t_{err\_m}$ A, $\lambda_{2}$ =1542.14 [nm]'))
  #lns_a.append(ax.errorbar(selected_x_49, selected_time_err_a_49, yerr = selected_time_err_a_49_stddev, fmt='^', capsize=6, color='C4',   label=r'$t_{err\_m}$ A, $\lambda_{2}$ =1528.77 [nm]'))
  lns_a.append(ax.errorbar(selected_x_7,  selected_time_err_a_7,  yerr = selected_time_err_a_7_stddev,  fmt='^', capsize=6, color='C5', label=r'$t_{err\_m}$ A, $\lambda_{2}$ =1563.05 [nm]'))
  lns_a.append(ax.errorbar(selected_x_8,  selected_time_err_a_8,  yerr = selected_time_err_a_8_stddev,  fmt='^', capsize=6, color='C6', label=r'$t_{err\_m}$ A, $\lambda_{2}$ =1562.23 [nm]'))
  lns_a.append(ax.errorbar(selected_x_9,  selected_time_err_a_9,  yerr = selected_time_err_a_9_stddev,  fmt='^', capsize=6, color='C7',  label=r'$t_{err\_m}$ A, $\lambda_{2}$ =1560.61 [nm]'))
  lns_a.append(ax.errorbar(selected_x_10, selected_time_err_a_10, yerr = selected_time_err_a_10_stddev, fmt='^', capsize=6, color='C8',   label=r'$t_{err\_m}$ A, $\lambda_{2}$ =1559.79 [nm]'))

  #lns_b.append(ax.errorbar(selected_x_0,  selected_time_err_b_0,  yerr = selected_time_err_b_0_stddev,  fmt='v', capsize=6, color='C1',    label=r'$t_{err\_m}$ B, $\lambda_{2}$ =1568.77 [nm]'))
  #lns_b.append(ax.errorbar(selected_x_16, selected_time_err_b_16, yerr = selected_time_err_b_16_stddev, fmt='v', capsize=6, color='C2',   label=r'$t_{err\_m}$ B, $\lambda_{2}$ =1554.94 [nm]'))
  #lns_b.append(ax.errorbar(selected_x_32, selected_time_err_b_32, yerr = selected_time_err_b_32_stddev, fmt='v', capsize=6, color='C3',   label=r'$t_{err\_m}$ B, $\lambda_{2}$ =1542.14 [nm]'))
  #lns_b.append(ax.errorbar(selected_x_49, selected_time_err_b_49, yerr = selected_time_err_b_49_stddev, fmt='v', capsize=6, color='C4',   label=r'$t_{err\_m}$ B, $\lambda_{2}$ =1528.77 [nm]'))
  lns_b.append(ax.errorbar(selected_x_7,  selected_time_err_b_7,  yerr = selected_time_err_b_7_stddev,  fmt='v', capsize=6, color='C5', label=r'$t_{err\_m}$ B, $\lambda_{2}$ =1563.05 [nm]'))
  lns_b.append(ax.errorbar(selected_x_8,  selected_time_err_b_8,  yerr = selected_time_err_b_8_stddev,  fmt='v', capsize=6, color='C6', label=r'$t_{err\_m}$ B, $\lambda_{2}$ =1562.23 [nm]'))
  lns_b.append(ax.errorbar(selected_x_9,  selected_time_err_b_9,  yerr = selected_time_err_b_9_stddev,  fmt='v', capsize=6, color='C7',  label=r'$t_{err\_m}$ B, $\lambda_{2}$ =1560.61 [nm]'))
  lns_b.append(ax.errorbar(selected_x_10, selected_time_err_b_10, yerr = selected_time_err_b_10_stddev, fmt='v', capsize=6, color='C8',   label=r'$t_{err\_m}$ B, $\lambda_{2}$ =1559.79 [nm]'))

  lns.append(ax.errorbar(x, time_err_calc_sel, yerr = 0, fmt='*', color='blue', label=r'$t_{err\_c}$ A'))
  lns.append(ax.errorbar(x, time_err_calc_3wl[0:10], yerr = time_err_calc_3wl_stddev[0:10], fmt='o', color='red', label=r'$t_{err\_c}$ B'))
  #############################################################################################

  labels = [l.get_label() for l in lns]
  legend1 = plt.legend(lns, labels, loc='upper left', fontsize='medium')

  lns_comb = lns_a + lns_b
  labels_comb = [l.get_label() for l in lns_comb]
  legend_comb = plt.legend(lns_comb, labels_comb, loc='upper right', ncol = 2, fontsize='medium')

  #labels = [r'3-$\lambda$ method III-B'] + [l.get_label() for l in lns_a]
  #labels = [l.get_label() for l in lns_a]
  #legend2= plt.legend(lns_a, labels, loc='lower left', fontsize='medium')

  #labels = [r'3-$\lambda$ method III-C'] + [l.get_label() for l in lns_b]
  #labels = [l.get_label() for l in lns_b]
  #legend3 = plt.legend(lns_b, labels, loc='upper right', fontsize='medium')

  ax.add_artist(legend1)
  ax.add_artist(legend_comb)
  #ax.add_artist(legend2)
  #ax.add_artist(legend3)

  ax.grid(linestyle='dotted')
  fig.subplots_adjust(left=0.17)

  #labels = [l.get_label() for l in lns]
  #ax.legend(lns, labels, loc='upper right', fontsize='medium')
    
  fig.savefig("time_error_small_wavelenfgth_offset.png")
  plt.show()

  fig.clf()
 
  ## The alpha plots with actual measured data are created in analyze_sellmeier.py
  '''
  ## alpha_sel(l1), alpha_stddev_sel(l1), alpha_3wl(l1), alpha_3wl_stddev(l1), time_err (i.e the calculated time error)
  ## data source: p:\App\ASTERICS_Cleopatra_WP5_1\WR_Calibration\wr-calibration\sw\insitu_alpha\data_180914\averaged_err.out
  ## time_err_b, time_err_b_stddev (i.e. PPS diff results)
  ## data source ./calc_diff.py data_180926/181030_15_19_37_DeltaDelay_meas data_180926/181031_09_12_18_DeltaDelay_meas -t 400 -o data_180914_pps
  ## data_180914_pps/190409_11_25_10_DeltaDelayDifference
  
  wavelength = numpy.array([1563.05, 1563.05, 1562.23, 1562.23, 1562.23, 1560.61, 1560.61, 1560.61, 1559.79, 1559.79])

  alpha_sel = numpy.array([6.424e-06, 6.424e-06, 3.207e-06, 3.207e-06, 3.207e-06, -3.197e-06, -3.197e-06, -3.197e-06, -6.384e-06, -6.384e-06])
  alpha_sel_stddev = numpy.array([1.682e-08, 1.682e-08, 1.674e-08, 1.674e-08, 1.674e-08, 1.671e-08, 1.671e-08, 1.671e-08, 1.675e-08, 1.675e-08])
  
  alpha_3wl_a = numpy.array([6.430e-06, 6.417e-06, 3.214e-06, 3.204e-06, 3.200e-06, -3.204e-06, -3.200e-06, -3.190e-06, -6.390e-06, -6.377e-06])
  alpha_3wl_a_stddev = numpy.array([1.431e-06, 4.783e-07, 7.155e-07, 3.591e-07, 2.400e-07, 2.391e-07, 3.588e-07, 7.203e-07, 4.786e-07, 1.439e-06])

  lns = []
  fig = plt.figure("Fiber Delay Coefficient")
  ax = fig.add_subplot(111)
  ax.set_title("Fiber Delay Coefficient")
  ax.set_ylabel(r'Delay coefficient ($\alpha$)')
  ax.set_xlabel(r'$\lambda_{1}$ Wavelength [nm]')
  lns.append(ax.errorbar(wavelength, alpha_3wl_a, yerr = alpha_3wl_a_stddev, fmt='*', capsize=6, color='red', label=r'3-$\lambda$'))
  lns.append(ax.errorbar(wavelength, alpha_sel, yerr = alpha_sel_stddev, fmt='*', capsize=6, color='green', label='Sellmeier'))

  labels = [l.get_label() for l in lns]
  ax.legend(lns, labels, loc='lower right', fontsize='medium')
    
  fig.savefig("t_alpha1.png")
  #plt.show()

  fig.clf()
  '''
 
  ##############################################################
  #### Full C-band wavelength offset
  ##############################################################

  #############################################################################################
  ## This piece of code plots the calculated time error (time_err_calc_sel) due to the difference of
  ## the "perfect" alpha (derived from the Sellmeier fit) and the imperfect alpha derived from
  ## the 3-lambda calculations on the same data set.
  ## (time_err_a, time_err_a_stddev) are the actual measured time errors due to this imperfect
  ## alpha.
  ## (time_err_b, time_err_b_stddev) are the actual measured time errors due to an imperfect
  ## alpha that was determined using the 3-Lambda probed CRTT measurements.
  #############################################################################################

  ## Small Lambda selection
  #l1_selection = [7, 7, 8, 8, 8, 9, 9, 9, 10, 10]
  #l2_selection = [8, 9, 7, 9, 10, 7, 8, 10, 8, 9] # determines color
  ## Full C-band selection
  l1_selection = [0, 0, 0, 16, 16, 16, 32, 32, 32, 49, 49, 49]
  l2_selection = [16, 32, 49, 0, 32, 49, 0, 16, 49, 0, 16, 32] # determines color
  ## concatenate both
  #l1_selection = l1_small_selection + l1_cband_selection
  #l2_selection = l2_small_selection + l2_cband_selection
  
  length = 12
  x = numpy.array(range(1, length+1))
  # time_err_calc_sel (i.e the calculated time error)
  # data source: p:\App\ASTERICS_Cleopatra_WP5_1\WR_Calibration\wr-calibration\sw\insitu_alpha\data_180914\averaged_err.out
  time_err_calc_sel = numpy.array([29.5, 91.0, 160.0, 29.5, -83.2, -146.1, 91.0, -83.2, -449.5, 159.9, -146.1, -449.5])

  # time_meas_sel, time_meas_sel_stddev (i.e. PPS diff results)
  # data source ./calc_diff.py data_180703_combined/00_000_180703_00_00_00_Combined_DeltaDelay_meas data_180629_combined/00_000_180627_00_00_00_Combined_DeltaDelay_meas -t 600 -o data_180914_pps
  # => data_180914_pps/190409_10_07_48_DeltaDelayDifference
  time_err_a = numpy.array([46.9, 98.7, 170.0, 44.8, -71.6, -151.5, 101.7, -84.4, -435.4, 172.9, -140.4, -437.1])
  time_err_a_stddev = numpy.array([57.6, 52.6, 56.5, 79.8, 53.7, 57.9, 67.6, 55.6, 59.3, 57.8, 61.0, 53.9])
  
  # time_meas_3wl, time_meas_3wl_stddev (i.e. PPS diff results)
  # data source ./calc_diff.py data_180926/181030_15_19_37_DeltaDelay_meas data_180926/181031_09_12_18_DeltaDelay_meas -t 400 -o data_180914_pps
  # => data_180914_pps/190409_11_25_10_DeltaDelayDifference
  time_err_b = numpy.array([119.1, 160.1, 220.7, 70.8, -31.9, -116.0, 136.8, -66.8, -341.5, 202.3, -134.3, -367.0])
  time_err_b_stddev = numpy.array([49.5, 64.8, 58.5, 82.7, 65.1, 39.3, 42.0, 57.3, 40.5, 56.2, 45.9, 28.1])

  selected_x_0 = []
  selected_time_err_a_0 = []
  selected_time_err_a_0_stddev = []
  selected_time_err_b_0 = []
  selected_time_err_b_0_stddev = []
  selected_x_7 = []
  selected_time_err_a_7 = []
  selected_time_err_a_7_stddev = []
  selected_time_err_b_7 = []
  selected_time_err_b_7_stddev = []
  selected_x_8 = []
  selected_time_err_a_8 = []
  selected_time_err_a_8_stddev = []
  selected_time_err_b_8 = []
  selected_time_err_b_8_stddev = []
  selected_x_9 = []
  selected_time_err_a_9 = []
  selected_time_err_a_9_stddev = []
  selected_time_err_b_9 = []
  selected_time_err_b_9_stddev = []
  selected_x_10 = []
  selected_time_err_a_10 = []
  selected_time_err_a_10_stddev = []
  selected_time_err_b_10 = []
  selected_time_err_b_10_stddev = []
  selected_x_16 = []
  selected_time_err_a_16 = []
  selected_time_err_a_16_stddev = []
  selected_time_err_b_16 = []
  selected_time_err_b_16_stddev = []
  selected_x_32 = []
  selected_time_err_a_32 = []
  selected_time_err_a_32_stddev = []
  selected_time_err_b_32 = []
  selected_time_err_b_32_stddev = []
  selected_x_49 = []
  selected_time_err_a_49 = []
  selected_time_err_a_49_stddev = []
  selected_time_err_b_49 = []
  selected_time_err_b_49_stddev = []
  
  for i in range(length):
    if l2_selection[i] == 0:
      #print("hit 0")
      selected_x_0.append(i + 1)
      selected_time_err_a_0.append(time_err_a[i])
      selected_time_err_a_0_stddev.append(time_err_a_stddev [i])
      selected_time_err_b_0.append(time_err_b[i])
      selected_time_err_b_0_stddev.append(time_err_b_stddev[i])
    elif l2_selection[i] == 7:
      #print("hit 7")
      selected_x_7.append(i + 1)
      selected_time_err_a_7.append(time_err_a[i])
      selected_time_err_a_7_stddev.append(time_err_a_stddev [i])
      selected_time_err_b_7.append(time_err_b[i])
      selected_time_err_b_7_stddev.append(time_err_b_stddev[i])
    elif l2_selection[i] == 8:
      #print("hit 8")
      selected_x_8.append(i + 1)
      selected_time_err_a_8.append(time_err_a[i])
      selected_time_err_a_8_stddev.append(time_err_a_stddev [i])
      selected_time_err_b_8.append(time_err_b[i])
      selected_time_err_b_8_stddev.append(time_err_b_stddev[i])
    elif l2_selection[i] == 9:
      #print("hit 9")
      selected_x_9.append(i + 1)
      selected_time_err_a_9.append(time_err_a[i])
      selected_time_err_a_9_stddev.append(time_err_a_stddev [i])
      selected_time_err_b_9.append(time_err_b[i])
      selected_time_err_b_9_stddev.append(time_err_b_stddev[i])
    elif l2_selection[i] == 10:
      #print("hit 10")
      selected_x_10.append(i + 1)
      selected_time_err_a_10.append(time_err_a[i])
      selected_time_err_a_10_stddev.append(time_err_a_stddev [i])
      selected_time_err_b_10.append(time_err_b[i])
      selected_time_err_b_10_stddev.append(time_err_b_stddev[i])
    elif l2_selection[i] == 16:
      #print("hit 16")
      selected_x_16.append(i + 1)
      selected_time_err_a_16.append(time_err_a[i])
      selected_time_err_a_16_stddev.append(time_err_a_stddev [i])
      selected_time_err_b_16.append(time_err_b[i])
      selected_time_err_b_16_stddev.append(time_err_b_stddev[i])
    elif l2_selection[i] == 32:
      #print("hit 32")
      selected_x_32.append(i + 1)
      selected_time_err_a_32.append(time_err_a[i])
      selected_time_err_a_32_stddev.append(time_err_a_stddev [i])
      selected_time_err_b_32.append(time_err_b[i])
      selected_time_err_b_32_stddev.append(time_err_b_stddev[i])
    elif l2_selection[i] == 49:
      #print("hit 49")
      selected_x_49.append(i + 1)
      selected_time_err_a_49.append(time_err_a[i])
      selected_time_err_a_49_stddev.append(time_err_a_stddev [i])
      selected_time_err_b_49.append(time_err_b[i])
      selected_time_err_b_49_stddev.append(time_err_b_stddev[i])
 
  lns = []
  lns_a = []
  lns_b = []
  fig = plt.figure("Time_error")
  ax = fig.add_subplot(111)
  ax.set_title("Full C-Band calculated and measured time errors")
  ax.set_ylabel("Time error [ps]")
  ax.set_xlabel("measurement number")

  lns_a.append(ax.errorbar(selected_x_0,  selected_time_err_a_0,  yerr = selected_time_err_a_0_stddev,  fmt='^', capsize=6, color='C1',    label=r'$t_{err\_m}$ A, $\lambda_{2}$ =1568.77 [nm]'))
  lns_a.append(ax.errorbar(selected_x_16, selected_time_err_a_16, yerr = selected_time_err_a_16_stddev, fmt='^', capsize=6, color='C2',   label=r'$t_{err\_m}$ A, $\lambda_{2}$ =1554.94 [nm]'))
  lns_a.append(ax.errorbar(selected_x_32, selected_time_err_a_32, yerr = selected_time_err_a_32_stddev, fmt='^', capsize=6, color='C3',   label=r'$t_{err\_m}$ A, $\lambda_{2}$ =1542.14 [nm]'))
  lns_a.append(ax.errorbar(selected_x_49, selected_time_err_a_49, yerr = selected_time_err_a_49_stddev, fmt='^', capsize=6, color='C4',   label=r'$t_{err\_m}$ A, $\lambda_{2}$ =1528.77 [nm]'))
  #lns_a.append(ax.errorbar(selected_x_7,  selected_time_err_a_7,  yerr = selected_time_err_a_7_stddev,  fmt='^', capsize=6, color='C5', label=r'$t_{err\_m}$ A, $\lambda_{2}$ =1563.05 [nm]'))
  #lns_a.append(ax.errorbar(selected_x_8,  selected_time_err_a_8,  yerr = selected_time_err_a_8_stddev,  fmt='^', capsize=6, color='C6', label=r'$t_{err\_m}$ A, $\lambda_{2}$ =1562.23 [nm]'))
  #lns_a.append(ax.errorbar(selected_x_9,  selected_time_err_a_9,  yerr = selected_time_err_a_9_stddev,  fmt='^', capsize=6, color='C7',  label=r'$t_{err\_m}$ A, $\lambda_{2}$ =1560.61 [nm]'))
  #lns_a.append(ax.errorbar(selected_x_10, selected_time_err_a_10, yerr = selected_time_err_a_10_stddev, fmt='^', capsize=6, color='C8',   label=r'$t_{err\_m}$ A, $\lambda_{2}$ =1559.79 [nm]'))

  lns_b.append(ax.errorbar(selected_x_0,  selected_time_err_b_0,  yerr = selected_time_err_b_0_stddev,  fmt='v', capsize=6, color='C1',    label=r'$t_{err\_m}$ B, $\lambda_{2}$ =1568.77 [nm]'))
  lns_b.append(ax.errorbar(selected_x_16, selected_time_err_b_16, yerr = selected_time_err_b_16_stddev, fmt='v', capsize=6, color='C2',   label=r'$t_{err\_m}$ B, $\lambda_{2}$ =1554.94 [nm]'))
  lns_b.append(ax.errorbar(selected_x_32, selected_time_err_b_32, yerr = selected_time_err_b_32_stddev, fmt='v', capsize=6, color='C3',   label=r'$t_{err\_m}$ B, $\lambda_{2}$ =1542.14 [nm]'))
  lns_b.append(ax.errorbar(selected_x_49, selected_time_err_b_49, yerr = selected_time_err_b_49_stddev, fmt='v', capsize=6, color='C4',   label=r'$t_{err\_m}$ B, $\lambda_{2}$ =1528.77 [nm]'))
  #lns_b.append(ax.errorbar(selected_x_7,  selected_time_err_b_7,  yerr = selected_time_err_b_7_stddev,  fmt='v', capsize=6, color='C5', label=r'$t_{err\_m}$ B, $\lambda_{2}$ =1563.05 [nm]'))
  #lns_b.append(ax.errorbar(selected_x_8,  selected_time_err_b_8,  yerr = selected_time_err_b_8_stddev,  fmt='v', capsize=6, color='C6', label=r'$t_{err\_m}$ B, $\lambda_{2}$ =1562.23 [nm]'))
  #lns_b.append(ax.errorbar(selected_x_9,  selected_time_err_b_9,  yerr = selected_time_err_b_9_stddev,  fmt='v', capsize=6, color='C7',  label=r'$t_{err\_m}$ B, $\lambda_{2}$ =1560.61 [nm]'))
  #lns_b.append(ax.errorbar(selected_x_10, selected_time_err_b_10, yerr = selected_time_err_b_10_stddev, fmt='v', capsize=6, color='C8',   label=r'$t_{err\_m}$ B, $\lambda_{2}$ =1559.79 [nm]'))

  lns.append(ax.errorbar(x, time_err_calc_sel, yerr = 0, fmt='*', color='blue', label=r'$t_{err\_c}$ A'))
  lns.append(ax.errorbar(x, time_err_calc_3wl[10:23], yerr = time_err_calc_3wl_stddev[10:23], fmt='o', color='red', label=r'$t_{err\_c}$ B'))
  #############################################################################################

  labels = [l.get_label() for l in lns]
  legend1 = plt.legend(lns, labels, loc='upper right', fontsize='medium')

  lns_comb = lns_a + lns_b
  labels_comb = [l.get_label() for l in lns_comb]
  legend_comb = plt.legend(lns_comb, labels_comb, loc='lower left', ncol = 2, fontsize='medium')

  #labels = [r'3-$\lambda$ method III-B'] + [l.get_label() for l in lns_a]
  #labels = [l.get_label() for l in lns_a]
  #legend2= plt.legend(lns_a, labels, loc='lower left', fontsize='medium')

  #labels = [r'3-$\lambda$ method III-C'] + [l.get_label() for l in lns_b]
  #labels = [l.get_label() for l in lns_b]
  #legend3 = plt.legend(lns_b, labels, loc='lower right', fontsize='medium')

  ax.add_artist(legend1)
  ax.add_artist(legend_comb)
  #ax.add_artist(legend2)
  #ax.add_artist(legend3)
  ax.grid(linestyle='dotted')
  fig.subplots_adjust(left=0.17)

  #labels = [l.get_label() for l in lns]
  #ax.legend(lns, labels, loc='lower left', fontsize='medium')

  fig.savefig("time_error_full_cband.png")
  plt.show()

  fig.clf()
 
  ## The alpha plots with actual measured data are created in analyze_sellmeier.py
  '''
  ## alpha_sel(l1), alpha_stddev_sel(l1), alpha_3wl(l1), alpha_3wl_stddev(l1), time_err (i.e the calculated time error)
  ## data source: p:\App\ASTERICS_Cleopatra_WP5_1\WR_Calibration\wr-calibration\sw\insitu_alpha\data_180914\averaged_err.out
  ## time_err_b, time_err_b_stddev (i.e. PPS diff results)
  ## data source ./calc_diff.py data_180926/181030_15_19_37_DeltaDelay_meas data_180926/181031_09_12_18_DeltaDelay_meas -t 400 -o data_180914_pps
  ## data_180914_pps/190409_11_25_10_DeltaDelayDifference
  
  wavelength = numpy.array([1568.77, 1568.77, 1568.77, 1554.94, 1554.94, 1554.94, 1542.14, 1542.14, 1542.14, 1528.77, 1528.77, 1528.77])

  alpha_sel = numpy.array([2.922e-05, 2.922e-05, 2.922e-05, -2.529e-05, -2.529e-05, -2.529e-05, -7.391e-05, -7.391e-05, -7.391e-05, -1.226e-04, -1.226e-04, -1.226e-04])
  alpha_sel_stddev = numpy.array([1.843e-08, 1.843e-08, 1.843e-08, 1.786e-08, 1.786e-08, 1.786e-08, 2.600e-08, 2.600e-08, 2.600e-08, 3.850e-08, 3.850e-08, 3.850e-08])
  
  alpha_3wl_a = numpy.array([2.898e-05, 2.848e-05, 2.792e-05, -2.553e-05, -2.462e-05, -2.410e-05, -7.466e-05, -7.324e-05, -7.025e-05, -1.239e-04, -1.214e-04, -1.190e-04])
  alpha_3wl_a_stddev = numpy.array([3.814e-07, 2.011e-07, 1.362e-07, 3.361e-07, 3.729e-07, 1.855e-07, 5.259e-07, 1.108e-06, 1.088e-06, 6.012e-07, 9.303e-07, 1.843e-06])

  lns = []
  fig = plt.figure("Fiber Delay Coefficient")
  ax = fig.add_subplot(111)
  ax.set_title("Fiber Delay Coefficient")
  ax.set_ylabel("Delay coefficient (alpha)")
  ax.set_xlabel("Wavelength [nm]")
  lns.append(ax.errorbar(wavelength, alpha_3wl_a, yerr = alpha_3wl_a_stddev, fmt='*', capsize=6, color='red', label=r'3-$\lambda$'))
  lns.append(ax.errorbar(wavelength, alpha_sel, yerr = alpha_sel_stddev, fmt='*', capsize=6, color='green', label='Sellmeier'))

  labels = [l.get_label() for l in lns]
  ax.legend(lns, labels, loc='lower right', fontsize='medium')
    
  fig.savefig("t_alpha2.png")
  #plt.show()

  fig.clf()
  '''
  
  sys.exit()
